@@ -16,6 +16,7 @@ from gantry.curators.models import (
 from gantry.curators.selectors import Selector
 from gantry.dataset.gantry_dataset import GantryDataset
 from gantry.dataset.main import get_dataset
+from gantry.exceptions import ClientNotInitialized
 
 logger = logging.getLogger(__name__)
 
@@ -26,11 +27,18 @@ class CuratorClient:
 
     @typechecked
     def list_curators(self, application_name: Optional[str] = None) -> List[str]:
-        """
-        List all curator names associated with an application.
+        """Returns a list of the names of all the curators in the
+        organization if no ``application_name`` is passed, and
+        all the curators associated with the provided application
+        if a name is passed.
+
+        Args:
+            application_name (str, optional): defaults to ``None``,
+                application must exist.
 
         Returns:
-            List[str]
+            ``List[str]``: A list of the curators either in the organization or
+            for a specific application.
         """
         return [
             curator_info.name
@@ -39,12 +47,7 @@ class CuratorClient:
 
     @typechecked
     def get_all_curators(self, application_name: Optional[str] = None) -> List["Curator"]:
-        """
-        Get all curators associated with an application.
 
-        Returns:
-            List[Curator]
-        """
         res = self._get_all_curator_infos(application_name=application_name)
         return [Curator.from_curator_info(self._api_client, curator_info) for curator_info in res]
 
@@ -54,10 +57,11 @@ class CuratorClient:
         Get a curator by name. If the curator does not exist, an exception will be raised.
 
         Args:
-            name (str): Curator name.
+            name (str): the name of the curator to be retrieved.
 
         Returns:
-            Curator: A Curator object.
+            :class:`gantry.curators.Curator`: A Curator object representing the curator
+            corresponding to the provided name.
         """
         res = self._api_client.request("GET", f"/api/v1/curator/{name}", raise_for_status=True)
         return Curator.from_curator_info(self._api_client, CuratorInfo.parse_obj(res["data"]))
@@ -97,8 +101,39 @@ class CuratorClient:
             ]
         return [CuratorInfo.parse_obj(curator) for curator in res]
 
+    def ping(self) -> bool:
+        """
+        Pings the API client server to check if it is alive.
+        Returns True if alive, False if there is an error during ping process.
+        """
+        try:
+            # Cannot use /healthz/* endpoints as those will be answered by nginx
+            # need to use /.
+            # See https://linear.app/gantry/issue/ENG-2978/revisit-ping-in-sdk
+            self._api_client.request("GET", "/api/ping", raise_for_status=True)
+            return True
+        except Exception as e:
+            logger.error(f"Error during ping: {e}")
+            return False
+
+    def ready(self) -> bool:
+        """
+        Checks whether the API is ready to receive traffic with the provided API Key.
+        """
+        try:
+            self._api_client.request("GET", "/api/v1/auth-ping", raise_for_status=True)
+            return True
+        except Exception as e:
+            logger.error(f"Error during api key check: {e}")
+            return False
+
 
 class Curator:
+    """
+    A class representing a user defined curator, where the user provides the
+    selection criteria using a list of selectors, as well as other metadata
+    needed to create and manage the creation jobs.
+    """
     def __init__(
         self,
         api_client: Optional[APIClient] = None,
@@ -117,7 +152,7 @@ class Curator:
         selectors to select rows from the dataset and write them to a curated dataset.
         The curated dataset is a new dataset that is created by the curator. The curator
         will create the curated dataset if it does not already exist, or update it if it
-        does exist.
+        does exist.f
 
         Instantiating a curator does not create the curator in Gantry. To create the curator,
         call `Curator.create` after instantiating the curator. Curators that have been created
@@ -161,11 +196,13 @@ class Curator:
         self._api_client: APIClient = (
             globals._API_CLIENT if api_client is None else api_client  # type: ignore
         )
+        if self._api_client is None:
+            raise ClientNotInitialized()
         self._id = id
         self._name = name
         self._curated_dataset_name = name if curated_dataset_name is None else curated_dataset_name
         self._application_name = application_name
-        self._start_on = datetime.datetime.now() if start_on is None else start_on
+        self._start_on = datetime.datetime.utcnow() if start_on is None else start_on
         self._curation_interval = curation_interval
         self._curate_past_intervals = curate_past_intervals
         self._created_at = created_at
@@ -258,7 +295,7 @@ class Curator:
                 name does not correspond to an existing dataset. Defaults to False.
 
         Raises:
-            ValueError: If curator id is not set.
+            ValueError: If curator ID is not set.
             ValueError: If no update parameters are provided.
 
         Returns:
@@ -316,7 +353,7 @@ class Curator:
     @classmethod
     def from_curator_info(cls, api_client: APIClient, curator_info: CuratorInfo) -> "Curator":
         """
-        Instantiates a Curator object from a CuratorInfo object.
+        :meta private: Instantiates a Curator object from a CuratorInfo object.
 
         Args:
             api_client (APIClient): _description_
