@@ -35,39 +35,44 @@ class GantryDatasetClient:
         self._workspace = str(Path(value).resolve())
         Path(self._workspace).mkdir(exist_ok=True)
 
-    def set_working_directory(self, working_dir: str):
+    def set_working_directory(self, working_dir: str) -> None:
         """
-        Set dataset client working directory. Your dataset will be pulled into this folder.
+        Set the  dataset client working directory. Your dataset will be pulled into this folder
+        when running ``pull()``.
 
         Args:
-            working_dir (str): absoluate path of the working directory
+            working_dir (str): absolute path of the directory used to store local copy of dataset.
+
+        Returns:
+            ``None``
         """
         self.working_directory = working_dir
 
     @typechecked
     def create_dataset(
-        self, name: str, bucket_name: Optional[str] = None, model_name: Optional[str] = None
+        self, name: str, bucket_name: Optional[str] = None, app_name: Optional[str] = None
     ) -> GantryDataset:
         """
-        Create dataset
+        Create a dataset in your organization with the provided name, optionally providing a
+        storing bucket if desired.
 
         Args:
             name (str): dataset name
             bucket_name (str): Provide bucket name if you want to use your own bucket. If not
                 provided we will use a gantry managed bucket.
-            model_name (Optional[str], optional): gantry application name which will be used to set
+            app_name (Optional[str], optional): gantry application name which will be used to set
                 dataset schema if provided.
 
         Returns:
-            GantryDataset
+            :class:`gantry.dataset.GantryDataset`: an object representing the crated dataset.
         """
         data = {
             "name": name,
         }
 
-        if model_name:
+        if app_name:
             res = self._api_client.request(
-                "GET", f"/api/v1/applications/{model_name}/schemas", raise_for_status=True
+                "GET", f"/api/v1/applications/{app_name}/schemas", raise_for_status=True
             )
             data["model_id"] = res["data"]["id"]
 
@@ -93,13 +98,13 @@ class GantryDatasetClient:
     @typechecked
     def get_dataset(self, name: str) -> GantryDataset:
         """
-        Get dataset object
+        Query Gantry for a dataset object corresponding to the provided name.
 
         Args:
-            name (str): dataset name
+            name (str): the name of the dataset to be retrieved.
 
         Returns:
-            GantryDataset
+            :class:`gantry.dataset.GantryDataset`: An object representing the dataset name.
         """
         try:
             res = self._api_client.request("GET", f"/api/v1/datasets/{name}", raise_for_status=True)
@@ -124,13 +129,14 @@ class GantryDatasetClient:
     @typechecked
     def list_dataset_versions(self, dataset_name: str) -> List[Dict[str, Any]]:
         """
-        List dataset version history
+        Provide a list of versions of this dataset, and metadata associated
+        with those versions.
 
         Args:
-            name (str): dataset name
+            name (str): the name of the dataset.
 
         Returns:
-            list of commit info
+            ``List[Dict[str, Any]]``: list of dictionaries with metadata.
         """
         dataset = self.get_dataset(dataset_name)
 
@@ -139,13 +145,14 @@ class GantryDatasetClient:
     @typechecked
     def list_datasets(self, include_deleted: bool = False) -> List[Dict[str, Any]]:
         """
-        List datasets
+        List datasets all datasets in the organizations' namespace.
 
         Args:
-            include_deleted (bool): whether to return deleted datasets, defaults to false
+            include_deleted (bool): include deleted datasets, defaults to ``False``.
 
         Returns:
-            List of dataset information
+            ``List[Dict[str, Any]]``: List of dictionaries, each representing one
+            dataset and associated metadata.
         """
         res = self._api_client.request(
             "GET",
@@ -159,13 +166,13 @@ class GantryDatasetClient:
     @typechecked
     def delete_dataset(self, name: str):
         """
-        Delete dataset
+        Delete the named dataset.
 
         Args:
-            name (str): dataset name
+            name (str): the name of the dataset to be deleted.
 
         Returns:
-            none
+            ``None``
         """
         dataset = self.get_dataset(name)
 
@@ -174,7 +181,10 @@ class GantryDatasetClient:
     def ping(self) -> bool:
         """
         Pings the API client server to check if it is alive.
-        Returns True if alive, False if there is an error during ping process.
+        Returns ``True`` if alive, ``False`` if there is an error during ping process.
+
+        Returns
+            ``bool``: flag indicating liveness.
         """
         try:
             # Cannot use /healthz/* endpoints as those will be answered by nginx
