@@ -1,36 +1,45 @@
+import contextlib
+import csv
 import functools
 import hashlib
 import io
 import json
 import logging
 import os
+import re
 import shutil
 import uuid
 from collections import defaultdict
 from dataclasses import asdict, dataclass
+from keyword import iskeyword
 from pathlib import Path
 from typing import IO, Any, Callable, Dict, List, Optional, Union
 
 import datasets
 import jinja2
 import pandas as pd
+import requests
 import yaml
 from typeguard import typechecked
 
 from gantry.api_client import APIClient
 from gantry.dataset import constants
 from gantry.exceptions import (
+    DataSchemaMismatchError,
     DatasetCommitNotFoundError,
     DatasetDeletedException,
     DatasetHeadOutOfDateException,
     DatasetNotFoundError,
     GantryRequestException,
+    NoTabularDataError,
 )
+from gantry.query.core.dataframe import GantryDataFrame
 from gantry.utils import (
     download_file_from_url,
     get_files_checksum,
     list_all_files,
     parse_s3_path,
+    read_lines_from_url,
     upload_file_to_url,
 )
 
@@ -124,6 +133,12 @@ class DatasetFileInfo:
 
 
 class GantryDataset:
+    """
+    A class representing a lightweight container around your data that
+    provides simple version control semantics for helping to make ML
+    pipelines reproducible.
+    """
+
     def __init__(
         self,
         api_client: APIClient,
@@ -134,6 +149,9 @@ class GantryDataset:
         dataset_s3_prefix: str,
         workspace: str,
     ):
+        """
+        :meta private:
+        """
         self.dataset_name = dataset_name
         self.workspace = workspace
         self._api_client = api_client
@@ -142,12 +160,71 @@ class GantryDataset:
         self._dataset_s3_prefix = dataset_s3_prefix
         self._aws_region = aws_region
 
+    def push_gantry_dataframe(self, gantrydf: GantryDataFrame, filename: str = ""):
+        """
+        This method will take a GantryDataFrame as input, and save its contents to a new file in
+        a Dataset. [TODO: add ability to specify custom version message]
+
+        Example usage:
+
+            >>> dataset = gdataset.get_dataset(dataset_name)
+            >>> os.listdir(workspace + dataset_name)
+            ['.dataset_metadata', 'dataset_config.yaml', 'README.md', 'tabular_manifests']
+            >>> os.listdir(workspace + dataset_name + '/tabular_manifests')
+            []
+            >>> gantrydf = gquery.query(
+            ...   application=application,
+            ...   start_time=start,
+            ...   end_time=end,
+            ...   version="1.1",
+            ... )
+            >>>
+            >>> _ = dataset.push_gantry_dataframe(gantrydf)
+            >>>
+            >>> _ = dataset.pull()
+            >>> os.listdir(workspace + dataset_name + '/tabular_manifests')
+            ['GantryDataAutoSave__4390b343-b802-4add-ab51-f86e53979c73_2023-02-13_12:09_to_2023-02-13_17:09_rows_0_69.csv']
+
+        Args:
+            gantrydf (GantryDataFrame): the GantryDataFrame that needs to be added to the dataset.
+            filename (str): the name to be given to the file with data from the GantryDataFrame.
+        Returns:
+            Latest commit info.
+        """
+        start_time = gantrydf.query_info.start_time
+        end_time = gantrydf.query_info.end_time
+        final_filename = filename
+        if filename == "":
+            final_filename = f"GantryDataAutoSave__{uuid.uuid4()}_{start_time.strftime('%Y-%m-%d_%H:%M')}_to_{end_time.strftime('%Y-%m-%d_%H:%M')}"  # noqa: E501
+        app_node_id = gantrydf.query_info.application_node_id
+        filters = gantrydf.filters
+        tags = gantrydf.tags
+        resp = self._api_client.request(
+            "POST",
+            f"/api/v1/datasets/{self._dataset_id}/raw_data",
+            json={
+                "model_id": app_node_id,
+                "raw_data_query_parameters": {
+                    "filters": filters,
+                    "tags": tags,
+                    "include_join_id": False,
+                    "hidden_columns": None,
+                    "start_time": start_time,
+                    "end_time": end_time,
+                },
+                "raw_data_csv_prefix": final_filename,
+            },
+            raise_for_status=True,
+        )
+        commit_info = resp["data"]
+        return self._prune_commit_info(commit_info)
+
     def list_versions(self) -> List[Dict[str, Any]]:
         """
         List dataset version history
 
         Returns:
-            List[Dict[str, Any]]: dataset versions from latest to earliest.
+            ``List[Dict[str, Any]]``: dataset versions from latest to earliest.
         """
         response = self._api_client.request(
             "GET", f"/api/v1/datasets/{self._dataset_id}/commits", raise_for_status=True
@@ -157,13 +234,19 @@ class GantryDataset:
     @_ensure_dataset_exists_locally
     def get_diff(self) -> Dict[str, List[str]]:
         """
-        Get local changes which hasn't been pushed
-        Return:
-            {
-                "new_files": List[str],
-                "modified_files": List[str],
-                "deleted_files": List[str],
-            }
+        Get lists of new, modified, and deleted files in your local
+        working directory.
+
+        Returns:
+            ``dict``: a dictionary representing the diff, which looks like this:
+
+            .. code-block:: json
+
+                {
+                    "new_files": List[str],
+                    "modified_files": List[str],
+                    "deleted_files": List[str],
+                }
         """
 
         diff = self._get_diff()
@@ -178,13 +261,15 @@ class GantryDataset:
     @_ensure_not_deleted
     def push_version(self, message: str) -> Dict[str, Any]:
         """
-        This method will take a snapshot of the dataset local change and create a new dataset
-        version.
+        Create a new version of this dataset that snapshots any new, modified, or deleted
+        files that exist in the local ``workspace``, a directory where a local copy of your
+        dataset lives, and pushes it to Gantry.
 
         Args:
-            message (str): version message, can't be empty
+            message (str): a non-empty string representing a message to associate with
+                the created version.
         Returns:
-            Version info
+            ``Dict[str, Any]``: a dictionary with metadata representing the version created.
         """
         commit_id = uuid.uuid4()
         diff = self._get_diff_for_commit(commit_id)
@@ -264,20 +349,21 @@ class GantryDataset:
         parent_version_id: Optional[Union[uuid.UUID, str]] = None,
     ) -> Dict[str, Any]:
         """
-        This method will upload a file (such as a file opened with ``open()``) to the dataset and
-        create a new dataset version. Does not rely on the user having the dataset synced,
-        nor does it cause the dataset to sync after the upload.
+        Create a new version of this dataset that contains a single new file passed as the
+        ``file`` parameter for this method, and returns information about the newly created
+        version.
 
         Args:
             file (IO): the file to be uploaded
-            filename (str, Optional): the name of the file to be uploaded. Defaults to the local
-                filename of the file
+            filename (str, Optional): the name of the file to be uploaded. Defaults to the
+                name of the file passed in the ``file`` parameter.
             message (str, Optional): the version message that will be associated with the
                 upload of the file. Defaults to a generic message if not set
             parent_version_id (uuid.UUID, Optional): If specified, SDK will check whether you are
                 making changes on top of the latest version, if not the operation will fail.
+
         Returns:
-            Version info
+            ``Dict[str, Any]``: A dictionary with metadata representing the version created.
         """
         if filename:
             name = filename
@@ -296,27 +382,28 @@ class GantryDataset:
         parent_version_id: Optional[Union[uuid.UUID, str]] = None,
     ) -> Dict[str, Any]:
         """
-        This method will upload a csv file (such as a csv file opened with ``open()``) to the
-        tabular_manifests of the dataset and create a new dataset version. Does not rely on the
-        user having the dataset synced, nor does it cause the dataset to sync after the upload.
+        Create a new version of this dataset that contains a single new CSV file passed as the
+        ``file`` parameter for this method, and returns information about the newly created
+        version. The filename must be postfixed with ``.csv``.
 
         Args:
             file (IO): the csv file to be uploaded
             filename (str, Optional): the name of the file to be uploaded. Defaults to the local
-                filename of the file. Name must end with '.csv'
-            message (str, Optional): the version message that will be associated with the
+                file name of the file. Name must end with ``.csv``.
+            message (str, optional): the version message that will be associated with the
                 upload of the file. Defaults to a generic message if not set
-            parent_version_id (uuid.UUID, Optional): If specified, SDK will check whether you are
+            parent_version_id (uuid.UUID, optional): If specified, SDK will check whether you are
                 making changes on top of the latest version, if not the operation will fail.
+
         Returns:
-            Version info
+            ``Dict[str, Any]``: A dictionary with metadata representing the version created.
         """
         if filename:
             name = filename
         else:
             name = os.path.basename(file.name)
 
-        if name[-4:] != ".csv":
+        if name[-4:] != constants.CSV_SUFFIX:
             raise ValueError(f"filename must end with '.csv'! Got filename: {name}")
 
         return self._add_single_file(file, name, message, parent_version_id)
@@ -331,32 +418,34 @@ class GantryDataset:
         parent_version_id: Optional[Union[uuid.UUID, str]] = None,
     ) -> Dict[str, Any]:
         """
-        This method will upload a pandas DataFrame in the form of a csv and create a new dataset
-        version. Does not rely on the user having the dataset synced, nor does it cause the dataset
-        to sync after the upload.
+        Create a new version of this dataset that contains a single new CSV created by calling
+        ``to_csv(tempfile, index=False)`` on the ``pandas.DataFrame`` passed as the ``dataframe``
+        parameter to this function. Optionally override the CSV name which will default to a
+        random hash.
 
         Args:
             dataframe (pd.DataFrame): the dataframe to be uploaded
             csvname (str, Optional): the name for the csv that will be uploaded. Defaults to
-                dataframe-[md5 hash].csv
+                dataframe-<random-hash>.csv
             message (str, Optional): the version message that will be associated with the
                 upload of the file. Defaults to a generic message if not set
             parent_version_id (uuid.UUID, Optional): If specified, SDK will check whether you are
                 making changes on top of the latest version, if not the operation will fail.
+
         Returns:
-            Version info
+            ``Dict[str, Any]``: A dictionary with metadata representing the version created.
         """
         if csvname:
-            if csvname[-4:] == ".csv":
+            if csvname[-4:] == constants.CSV_SUFFIX:
                 name = csvname
             else:
-                name = f"{csvname}.csv"
+                name = f"{csvname}{constants.CSV_SUFFIX}"
         else:
             hash = hashlib.md5(pd.util.hash_pandas_object(dataframe).values).hexdigest()
-            name = f"dataframe-{hash}.csv"
+            name = f"dataframe-{hash}{constants.CSV_SUFFIX}"
 
         file = io.StringIO()
-        dataframe.to_csv(file)
+        dataframe.to_csv(file, index=False)
         file.seek(0)
 
         return self._add_single_file(file, name, message, parent_version_id)
@@ -368,14 +457,16 @@ class GantryDataset:
         self, version_id: Union[str, uuid.UUID], forced: bool = False
     ) -> Optional[Dict[str, Any]]:
         """
-        Roll back the dataset to a previous version based on version id, then pull the target
-        version to the local folder.
+        Roll back the dataset to a previous version based on version ID, then pull the target
+        version to the local directory specified in ``workspace``.
 
         Args:
-            version_id (str): target version id
-            forced (bool): whether to ignore the presence of any local changes
+            version_id (str): ID of the version to rollback to.
+            forced (bool): whether to discard local changes when rolling back.
+
         Returns:
-            Version info (optional): version info if rollback was successsful.
+            Optional[Dict[str, Any]]: A dictionary with metadata representing the versioned
+            rolled back to if successful, ``None`` otherwise.
         """
         target_commit = self._get_commit(version_id)
 
@@ -407,13 +498,14 @@ class GantryDataset:
         accidentally losing changes
 
         Args:
-            version_id (str, optional): target version id. Defaults to None.
+            version_id (str, optional): target version ID, defaults to ``None`` which will pull the
+                latest version.
             forced (bool): whether to ignore the presence of any local changes
-        Returns:
-            version info (optional): version info if pull was successful. None if there were local
-                changes
-        """
 
+        Returns:
+            Optional[Dict[str, Any]]: A dictionary with metadata representing the versioned rolled
+            back to if successful, ``None`` otherwise.
+        """
         target_commit = (
             self._get_commit(commit_id=version_id) if version_id else self._get_latest_commit()
         )
@@ -442,18 +534,31 @@ class GantryDataset:
         if not os.path.exists(self._get_file_path(constants.STASH_FOLDER)):
             os.mkdir(self._get_file_path(constants.STASH_FOLDER))
 
+        # Create the tabular_manifests and artifact folder
+        (Path(self.workspace) / self.dataset_name / constants.TABULAR_MANIFESTS).resolve().mkdir(
+            parents=True, exist_ok=True
+        )
+
+        (Path(self.workspace) / self.dataset_name / constants.ARTIFACTS).resolve().mkdir(
+            parents=True, exist_ok=True
+        )
+
         return self._prune_commit_info(target_commit)
 
     def stash(self) -> None:
         """
-        Stashes local changes
+        Stashes local changes for later use.
+
+        Returns:
+            ``None``
         """
         # deletes existing stash
         if os.path.exists(self._get_file_path(constants.DATASET_STASH_FILE)):
             with open(self._get_file_path(constants.DATASET_STASH_FILE)) as stashfile:
                 old_stash = json.load(stashfile)
             for file in old_stash[constants.MODIFIED_FILES] + old_stash[constants.NEW_FILES]:
-                os.remove(self._get_file_path(f"{constants.STASH_FOLDER}/{file}"))
+                with contextlib.suppress(FileNotFoundError):  # file not exist is OK
+                    os.remove(self._get_file_path(f"{constants.STASH_FOLDER}/{file}"))
 
         # creates new stash
         local_diff = self.get_diff()
@@ -478,7 +583,10 @@ class GantryDataset:
 
     def restore(self) -> None:
         """
-        Restores changes from stash
+        Restore changes from local stash.
+
+        Returns:
+            ``None``
         """
         if not os.path.exists(self._get_file_path(constants.DATASET_STASH_FILE)):
             logger.warning("No stashed files were found!")
@@ -500,7 +608,7 @@ class GantryDataset:
 
     def delete(self):
         """
-        Deletes the dataset (currently it is just marked as deleted)
+        Soft delete this dataset.
         """
         self._api_client.request(
             "DELETE",
@@ -771,16 +879,20 @@ class GantryDataset:
 
     def _stash_file(self, file_path: str):
         # move file to stash folder
+        dest_path = self._get_file_path(f"{constants.STASH_FOLDER}/{file_path}")
+        os.makedirs(Path(dest_path).parent, exist_ok=True)
         os.replace(
             self._get_file_path(file_path),
-            self._get_file_path(f"{constants.STASH_FOLDER}/{file_path}"),
+            dest_path,
         )
 
     def _restore_stash_file(self, file_path: str):
         # recover file from stash folder
+        dest_path = self._get_file_path(file_path)
+        os.makedirs(Path(dest_path).parent, exist_ok=True)
         os.replace(
             self._get_file_path(f"{constants.STASH_FOLDER}/{file_path}"),
-            self._get_file_path(file_path),
+            dest_path,
         )
 
     def _download_files(self, file_list: List[DatasetFileInfo]):
@@ -792,27 +904,30 @@ class GantryDataset:
         """
         for i in range(0, len(file_list), BATCH_SIZE):
             batch = file_list[i : i + BATCH_SIZE]
-            resp = self._api_client.request(
-                "POST",
-                f"/api/v1/datasets/{self._dataset_id}/presign/getobject",
-                json={
-                    "obj_infos": [
-                        {
-                            constants.OBJ_KEY: self._get_obj_key(item.url),
-                            constants.VERSION_ID: item.version_id,
-                        }
-                        for item in batch
-                    ],
-                },
-                raise_for_status=True,
-            )
-            presigned_urls = resp["data"]
+            presigned_urls = self._generate_get_presign_url(batch=batch)
 
             for item in batch:
                 local_path = self._get_file_path(item.file_name)
                 presigned_url = presigned_urls[self._get_obj_key(item.url)]
                 download_file_from_url(presigned_url=presigned_url, local_path=Path(local_path))
 
+    def _generate_get_presign_url(self, batch: List[DatasetFileInfo]):
+        resp = self._api_client.request(
+            "POST",
+            f"/api/v1/datasets/{self._dataset_id}/presign/getobject",
+            json={
+                "obj_infos": [
+                    {
+                        constants.OBJ_KEY: self._get_obj_key(item.url),
+                        constants.VERSION_ID: item.version_id,
+                    }
+                    for item in batch
+                ],
+            },
+            raise_for_status=True,
+        )
+        return resp["data"]
+
     def _upload_files(self, file_list: List[DatasetFileInfo]):
         """
         download file to the specific path
@@ -850,13 +965,154 @@ class GantryDataset:
             A key value pair of Dict where key is the column name and value is huggingface
             feature type
         """
-        columns = dict()
         config = yaml.safe_load(
             (Path(self.workspace) / self.dataset_name / constants.DATASET_CONFIG_FILE).open(
                 mode="r"
             )
         )
+        return self._get_feature_map_from_config(config=config)
 
+    def pull_huggingface_dataset_by_version(
+        self,
+        version_id: Optional[str] = None,
+        hf_dataset_name: Optional[str] = None,
+        streaming=False,
+    ):
+        """
+        This function will pull the dataset tabular files from gantry server and convert it
+        to huggingface dataset. This operation will not change your local dataset folder, it
+        will only load dataset tabular files exist in the version you specified into a
+        huggingface dataset object.
+
+        Args:
+            version_id (Optional[str], optional): dataset version id to load. Defaults to latest.
+            hf_dataset_name (Optional[str], optional): param to overwrite the huggingface
+                                                dataset name. Note this param must be a valid
+                                                python identifier.
+            streaming (bool, default ``False``): If set to True, don't download the data files.
+                                                 Instead, it streams the data progressively while
+                                                 iterating on the dataset. An IterableDataset or
+                                                 IterableDatasetDict is returned instead
+                                                 in this case.
+        """
+        if not hf_dataset_name:
+            hf_dataset_name = self._clean_variable_name(self.dataset_name)
+
+        if not GantryDataset._is_valid_python_identifier(hf_dataset_name=hf_dataset_name):
+            return
+
+        manifest_presign_url = self._get_manifest_presign_url(version_id=version_id)
+        tabular_files = self._get_tabular_files(manifest_presign_url=manifest_presign_url)
+        features = self._extract_feature_schema_from_s3(manifest_presign_url=manifest_presign_url)
+        if not features:
+            raise DataSchemaMismatchError()
+        training_files = []
+        for i in range(0, len(tabular_files), BATCH_SIZE):
+            batch = tabular_files[i : i + BATCH_SIZE]
+            presigned_urls = self._generate_get_presign_url(batch=batch)
+            training_files.extend(presigned_urls.values())
+
+        if not training_files:
+            raise NoTabularDataError()
+
+        env = jinja2.Environment(
+            loader=jinja2.FileSystemLoader(
+                os.path.join(os.path.dirname(os.path.realpath(__file__)), "templates")
+            )
+        )
+
+        template = env.get_template("load_script_template.py.jinja")
+
+        content = template.render(
+            training_file_list=training_files,
+            dataset_version="1.0.0",  # Format should be x.y.z with {{x,y,z}} being digits
+            dataset_name=hf_dataset_name,
+            features=features,
+        )
+        hf_loading_script = (
+            Path(self.workspace) / self.dataset_name / constants.HF_FOLDER / f"{hf_dataset_name}.py"
+        )
+        hf_loading_script.parent.mkdir(parents=True, exist_ok=True)
+        with hf_loading_script.open("w") as f:
+            f.write(content)
+
+        try:
+            hf_dataset = datasets.load_dataset(
+                str(hf_loading_script), streaming=streaming, split="train"
+            )
+        except datasets.builder.DatasetGenerationError as e:
+            raise DataSchemaMismatchError from e
+
+        return hf_dataset
+
+    def _get_manifest_presign_url(self, version_id: Optional[str] = None) -> str:
+        target_commit = (
+            self._get_commit(commit_id=version_id) if version_id else self._get_latest_commit()
+        )
+
+        metadata_version_id = target_commit[constants.METADATA_S3_FILE_VERSION]
+        manifest_s3_url = self._get_s3_url(constants.DATASET_MANIFEST_FILE)
+
+        return self._generate_get_presign_url(
+            [
+                DatasetFileInfo(
+                    file_name=constants.DATASET_MANIFEST_FILE,
+                    url=manifest_s3_url,
+                    version_id=metadata_version_id,
+                )
+            ]
+        )[self._get_obj_key(manifest_s3_url)]
+
+    def _get_tabular_files(self, manifest_presign_url: str) -> List[DatasetFileInfo]:
+        """
+        Get tabular file list based on the version id
+
+        Args:
+            version_id (Optional[str], optional): dataset version. Defaults to None.
+
+        Returns:
+            List[DatasetFileInfo]: tabular file list
+        """
+        tabular_files = []
+        for line in read_lines_from_url(manifest_presign_url):
+            file_info = json.loads(line)
+            if file_info[constants.FILE_NAME].startswith(constants.TABULAR_MANIFESTS) and file_info[
+                constants.FILE_NAME
+            ].endswith(constants.CSV_SUFFIX):
+                tabular_files.append(
+                    DatasetFileInfo(
+                        file_name=file_info[constants.FILE_NAME],
+                        url=file_info[constants.URL],
+                        version_id=file_info[constants.VERSION_ID],
+                    )
+                )
+        return tabular_files
+
+    def _extract_feature_schema_from_s3(self, manifest_presign_url: str):
+        for line in read_lines_from_url(manifest_presign_url):
+            file_info = json.loads(line)
+            if file_info[constants.FILE_NAME] == constants.DATASET_CONFIG_FILE:
+                # read config file
+                # generate the feature map
+                config_presign_url = self._generate_get_presign_url(
+                    [
+                        DatasetFileInfo(
+                            file_name=file_info[constants.FILE_NAME],
+                            url=file_info[constants.URL],
+                            version_id=file_info[constants.VERSION_ID],
+                        )
+                    ]
+                )[self._get_obj_key(file_info[constants.URL])]
+
+                # generate presign url for config file
+                response = requests.get(config_presign_url)
+                config = yaml.safe_load(response.text)
+
+                return GantryDataset._get_feature_map_from_config(config=config)
+
+    @staticmethod
+    def _get_feature_map_from_config(config):
+        columns = dict()
         if constants.DATASET_FEATURES_KEY in config:
             for feature_name, feature_dtype in config[constants.DATASET_FEATURES_KEY].items():
                 columns[feature_name] = constants.GANTRY_2_HF_DTYPE[feature_dtype]
@@ -867,14 +1123,42 @@ class GantryDataset:
 
         return columns
 
-    def get_huggingface_dataset(self):
+    def get_huggingface_dataset(
+        self, hf_dataset_name: Optional[str] = None, streaming: bool = False
+    ):
         """
         Create a huggingface dataset based on the local dataset files
 
         This method will create a huggingface load script based on the dataset config and
         then load the dataset into huggingface pyarrow dataset.
+
+        Args:
+            versoion_id (Optional[str], optional): dataset version id to load. Defaults to latest.
+            hf_dataset_name (Optional[str], optional): param to overwrite the huggingface
+                                                dataset name. Note this param must be a valid
+                                                python identifier.
+            streaming (bool, default ``False``): If set to True, don't download the data files.
+                                                 Instead, it streams the data progressively while
+                                                 iterating on the dataset. An IterableDataset or
+                                                 IterableDatasetDict is returned instead
+                                                 in this case.
         """
 
+        if not hf_dataset_name:
+            hf_dataset_name = GantryDataset._clean_variable_name(self.dataset_name)
+
+        if not GantryDataset._is_valid_python_identifier(hf_dataset_name=hf_dataset_name):
+            return
+
+        tabular_manifests_path = (
+            Path(self.workspace) / self.dataset_name / constants.TABULAR_MANIFESTS
+        ).resolve()
+
+        if not os.path.exists(tabular_manifests_path) or not any(
+            Path(tabular_manifests_path).iterdir()
+        ):
+            raise NoTabularDataError()
+
         env = jinja2.Environment(
             loader=jinja2.FileSystemLoader(
                 os.path.join(os.path.dirname(os.path.realpath(__file__)), "templates")
@@ -883,30 +1167,53 @@ class GantryDataset:
 
         template = env.get_template("load_script_template.py.jinja")
 
-        training_files = [
-            str(p)
-            for p in list_all_files(
-                (Path(self.workspace) / self.dataset_name / constants.TABULAR_MANIFESTS).resolve()
-            )
-        ]
+        training_files = [str(p) for p in list_all_files(tabular_manifests_path.resolve())]
+        features = self._extract_feature_type()
+
+        # verify the csvs agree with the schema
+        for csv_path in training_files:
+            with open(csv_path) as data_csv:
+                cols = next(csv.reader(data_csv))
+
+                if set(cols) != set(features):
+                    raise DataSchemaMismatchError()
 
         content = template.render(
             training_file_list=training_files,
             dataset_version="1.0.0",  # Format should be x.y.z with {{x,y,z}} being digits
-            dataset_name=self.dataset_name,
-            features=self._extract_feature_type(),
+            dataset_name=hf_dataset_name,
+            features=features,
         )
         hf_loading_script = (
-            Path(self.workspace)
-            / self.dataset_name
-            / constants.HF_FOLDER
-            / f"{self.dataset_name}.py"
+            Path(self.workspace) / self.dataset_name / constants.HF_FOLDER / f"{hf_dataset_name}.py"
         )
         os.makedirs(hf_loading_script.parent, exist_ok=True)
         with hf_loading_script.open("w") as f:
             f.write(content)
 
-        return datasets.load_dataset(str(hf_loading_script))
+        try:
+            hf_dataset = datasets.load_dataset(
+                str(hf_loading_script), streaming=streaming, split="train"
+            )
+        except datasets.builder.DatasetGenerationError as e:
+            raise DataSchemaMismatchError from e
+
+        return hf_dataset
+
+    @staticmethod
+    def _is_valid_python_identifier(hf_dataset_name):
+        if (
+            (not hf_dataset_name)
+            or (not hf_dataset_name.isidentifier())
+            or iskeyword(hf_dataset_name)
+        ):
+            logger.error(
+                "Can't get a valid hugging face dataset name, the name has to be a valid python"
+                + "identifier. Please use hf_dataset_name param to set a valid name and try again."
+            )
+            return False
+        else:
+            return True
 
     def _prune_commit_info(self, commit_info):
         return {
@@ -919,3 +1226,11 @@ class GantryDataset:
             "created_by": commit_info["created_by"],
             "is_latest_version": commit_info["is_latest_commit"],
         }
+
+    @staticmethod
+    def _clean_variable_name(ds_name):
+        # Remove invalid characters
+        ds_name = re.sub("[^0-9a-zA-Z_]", "", ds_name)
+        # Remove leading characters until we find a letter or underscore
+        ds_name = re.sub("^[^a-zA-Z_]+", "", ds_name)
+        return ds_name
