@@ -58,13 +58,13 @@ class GantryRequestException(GantryException):
             if "/api/v1/ingest/data-connectors" in url:
                 msg = (
                     "Data connector not found. Ensure the parameters refer to an existing "
-                    "data connector you have access to."
-                )
+                    "data connector you have access to. Details: {}"
+                ).format(response_text)
             else:
                 msg = (
-                    "Resource not found ({}). Ensure the parameters refer to an existing resource "
-                    "you have access to."
-                ).format(url)
+                    "A resource was not found while trying to access ({}). Ensure the parameters "
+                    "refer to an existing resource you have access to. Details: {}"
+                ).format(url, response_text)
         elif status_code == 429:
             if "/api/v1/ingest/data-connectors" in url:
                 msg = (
@@ -130,3 +130,30 @@ class DatasetHeadOutOfDateException(GantryException):
     def __init__(self):
         message = "Local HEAD not up to date! Your local version is behind the remote"
         super().__init__(message)
+
+
+class NoTabularDataError(GantryException):
+    """
+    Raised when there is no tabular data folder in a dataset
+    """
+
+    def __init__(self):
+        message = (
+            "No tabular data found in tabular_manifests folder! Please make sure that the"
+            "tabular_manifests folder exists and contains at least one manifest file."
+        )
+        super().__init__(message)
+
+
+class DataSchemaMismatchError(GantryException):
+    """
+    Raised when the dataset config schema disagrees with the csv content
+    """
+
+    def __init__(self):
+        message = (
+            "The dataset config schema disagrees with the data! "
+            "Ensure that the columns in the dataset config match those in the tabular data files "
+            "and that the types in the config are all correct."
+        )
+        super().__init__(message)
