@@ -1,3 +1,4 @@
+import datetime
 import json
 import logging
 import os
@@ -5,10 +6,11 @@ import uuid
 from io import StringIO
 from pathlib import Path
 
+import datasets
 import pandas as pd
 import pytest
 import responses
-from datasets import Value
+from datasets import Sequence, Value, builder
 from mock import patch
 from responses import matchers
 
@@ -23,12 +25,15 @@ from gantry.dataset.constants import (
 )
 from gantry.dataset.gantry_dataset import DatasetFileInfo, GantryDataset
 from gantry.exceptions import (
+    DataSchemaMismatchError,
     DatasetCommitNotFoundError,
     DatasetHeadOutOfDateException,
     DatasetNotFoundError,
     GantryException,
     GantryRequestException,
+    NoTabularDataError,
 )
+from gantry.query.core.dataframe import GantryDataFrame
 from gantry.utils import get_files_checksum, list_all_files
 
 from .conftest import (
@@ -173,18 +178,244 @@ def test_create_commit_without_download(gantry_dataset_obj):
         gantry_dataset_obj.push_version("test commit")
 
 
-def test_get_huggingface_dataset(gantry_dataset_obj):
+@patch(
+    "gantry.dataset.gantry_dataset.GantryDataset._get_manifest_presign_url",
+    return_value="s3://manifest/presign/url",
+)
+@patch("gantry.dataset.gantry_dataset.GantryDataset._get_tabular_files", return_value=[])
+@patch(
+    "gantry.dataset.gantry_dataset.GantryDataset._extract_feature_schema_from_s3", return_value={}
+)
+def test_pull_huggingface_dataset_by_version_no_schema(
+    mock_get_presign_url,
+    mock_get_tabular_files,
+    mock_extract_feature_schema_from_s3,
+    gantry_dataset_obj,
+):
+    """
+    Test pull huggingface dataset by version
+    """
+    with pytest.raises(DataSchemaMismatchError):
+        gantry_dataset_obj.pull_huggingface_dataset_by_version(
+            hf_dataset_name="hf_test",
+            version_id="1.0.0",
+        )
+
+
+@patch(
+    "gantry.dataset.gantry_dataset.GantryDataset._get_manifest_presign_url",
+    return_value="s3://manifest/presign/url",
+)
+@patch("gantry.dataset.gantry_dataset.GantryDataset._get_tabular_files", return_value=[])
+@patch(
+    "gantry.dataset.gantry_dataset.GantryDataset._extract_feature_schema_from_s3",
+    return_value={"feature": "schema"},
+)
+def test_pull_huggingface_dataset_by_version_no_data(
+    mock_get_presign_url,
+    mock_get_tabular_files,
+    mock_extract_feature_schema_from_s3,
+    gantry_dataset_obj,
+):
+    """
+    Test pull huggingface dataset by version
+    """
+    with pytest.raises(NoTabularDataError):
+        gantry_dataset_obj.pull_huggingface_dataset_by_version(
+            hf_dataset_name="hf_test",
+            version_id="1.0.0",
+        )
+
+
+@pytest.mark.parametrize("correct_schema, streaming", [(True, True), (True, False), (False, False)])
+@patch(
+    "gantry.dataset.gantry_dataset.GantryDataset._get_manifest_presign_url",
+    return_value="s3://manifest/presign/url",
+)
+@patch("gantry.dataset.gantry_dataset.GantryDataset._get_tabular_files")
+@patch("gantry.dataset.gantry_dataset.GantryDataset._extract_feature_schema_from_s3")
+@patch("gantry.dataset.gantry_dataset.GantryDataset._generate_get_presign_url")
+def test_pull_huggingface_dataset_by_version(
+    mock_generate_get_presign_url,
+    mock_extract_feature_schema_from_s3,
+    mock_get_tabular_files,
+    mock_get_manifest_presign_url,
+    correct_schema,
+    streaming,
+    gantry_dataset_obj,
+    datadir,
+):
+    """
+    Test pull huggingface dataset by version
+    """
+
+    if correct_schema:
+        mock_extract_feature_schema_from_s3.return_value = {
+            "inputs.credit_history": Value(dtype="string", id=None),
+            "inputs.credit_scores": Sequence(
+                feature=Value(dtype="float64", id=None), length=-1, id=None
+            ),
+            "inputs.description": Value(dtype="string", id=None),
+            "inputs.image_uri": Value(dtype="string", id=None),
+            "inputs.income": Value(dtype="int64", id=None),
+            "inputs.loan_id": Value(dtype="string", id=None),
+            "inputs.prediction": Value(dtype="bool", id=None),
+            "inputs.timestamp": Value(dtype="timestamp[ns, tz=UTC]", id=None),
+            "outputs.output": Value(dtype="float64", id=None),
+            "feedback.loan_repaid": Value(dtype="bool", id=None),
+            "__time": Value(dtype="timestamp[ns, tz=UTC]", id=None),
+        }
+    else:
+        mock_extract_feature_schema_from_s3.return_value = {
+            "inputs.credit_history": Value(dtype="string", id=None),
+            "inputs.prediction": Value(dtype="bool", id=None),
+            "inputs.timestamp": Value(dtype="timestamp[ns, tz=UTC]", id=None),
+            "outputs.output": Value(dtype="float64", id=None),
+            "feedback.loan_repaid": Value(dtype="bool", id=None),
+            "__time": Value(dtype="timestamp[ns, tz=UTC]", id=None),
+        }
+
+    def side_effect(**kwargs):
+        return {f.file_name: f.url for f in kwargs["batch"]}
+
+    mock_get_tabular_files.return_value = [
+        DatasetFileInfo(
+            file_name="mock_tabular_file_1.csv",
+            # for unit test we use local file path instead of s3 url
+            url=str(datadir / "hf-test" / "tabular_manifests" / "hf_dataset_1.csv"),
+            version_id="mock_version_id",
+        ),
+        DatasetFileInfo(
+            file_name="mock_tabular_file_2.csv",
+            url=str(datadir / "hf-test" / "tabular_manifests" / "hf_dataset_2.csv"),
+            version_id="mock_version_id",
+        ),
+    ]
+    mock_generate_get_presign_url.side_effect = side_effect
+
+    if correct_schema:
+        hf_dataset = gantry_dataset_obj.pull_huggingface_dataset_by_version(
+            version_id="mock_version_id", hf_dataset_name="hf_test", streaming=streaming
+        )
+        if not streaming:
+            assert len(hf_dataset) == 40
+        else:
+            assert len(list(hf_dataset)) == 40
+            hf_dataset.__class__ = datasets.iterable_dataset.IterableDataset
+
+        assert hf_dataset.features == {
+            "inputs.credit_history": Value(dtype="string", id=None),
+            "inputs.credit_scores": Sequence(
+                feature=Value(dtype="float64", id=None), length=-1, id=None
+            ),
+            "inputs.description": Value(dtype="string", id=None),
+            "inputs.image_uri": Value(dtype="string", id=None),
+            "inputs.income": Value(dtype="int64", id=None),
+            "inputs.loan_id": Value(dtype="string", id=None),
+            "inputs.prediction": Value(dtype="bool", id=None),
+            "inputs.timestamp": Value(dtype="timestamp[ns, tz=UTC]", id=None),
+            "outputs.output": Value(dtype="float64", id=None),
+            "feedback.loan_repaid": Value(dtype="bool", id=None),
+            "__time": Value(dtype="timestamp[ns, tz=UTC]", id=None),
+        }
+        assert hf_dataset.builder_name == "hf_test"
+    else:
+        with pytest.raises(DataSchemaMismatchError):
+            hf_dataset = gantry_dataset_obj.pull_huggingface_dataset_by_version(
+                version_id="mock_version_id", hf_dataset_name="hf_test"
+            )
+
+
+def test_get_tabular_files(gantry_dataset_obj, datadir):
+    with responses.RequestsMock() as resp:
+        MOCK_URL = "http://mock-url"
+        manifest_file = datadir / "hf-test" / ".dataset_metadata" / ".gantry_manifest.jsonl"
+        resp.add(responses.GET, MOCK_URL, body=manifest_file.read_text())
+        tabular_files = gantry_dataset_obj._get_tabular_files(MOCK_URL)
+        assert len(tabular_files) == 2
+        assert tabular_files[0].file_name.endswith(".csv")
+        assert tabular_files[0].file_name.endswith(".csv")
+
+
+@patch("gantry.dataset.gantry_dataset.GantryDataset._get_obj_key", return_value="mock_obj")
+@patch(
+    "gantry.dataset.gantry_dataset.GantryDataset._generate_get_presign_url",
+    return_value={"mock_obj": "http://mock-config-url"},
+)
+def test_extract_feature_schema_from_s3(
+    mock_generate_get_presign_url, mock_get_obj_key, gantry_dataset_obj, datadir
+):
+    MANIFEST_URL = "http://mock-manifest-url"
+    with responses.RequestsMock() as resp:
+        manifest_file = datadir / "hf-test" / ".dataset_metadata" / ".gantry_manifest.jsonl"
+        resp.add(responses.GET, MANIFEST_URL, body=manifest_file.read_text())
+        config_file = datadir / "hf-test" / "dataset_config.yaml"
+        resp.add(responses.GET, "http://mock-config-url", body=config_file.read_text())
+        features = gantry_dataset_obj._extract_feature_schema_from_s3(MANIFEST_URL)
+        assert len(features) == 11
+
+
+@pytest.mark.parametrize("use_latest_commit", [True, False])
+@patch("gantry.dataset.gantry_dataset.GantryDataset._get_commit")
+@patch("gantry.dataset.gantry_dataset.GantryDataset._get_latest_commit")
+@patch("gantry.dataset.gantry_dataset.GantryDataset._generate_get_presign_url")
+def test_get_manifest_presign_url(
+    mock_generate_get_presign_url,
+    mock_get_latest_commit,
+    mock_get_commit,
+    use_latest_commit,
+    commit_history,
+    gantry_dataset_obj,
+):
+    mock_get_latest_commit.return_value = commit_history[0]
+    mock_get_commit.return_value = commit_history[1]
+
+    def side_effect(args):
+        return {gantry_dataset_obj._get_obj_key(args[0].url): f"{args[0].url}-presign"}
+
+    mock_generate_get_presign_url.side_effect = side_effect
+    if use_latest_commit:
+        presign_url = gantry_dataset_obj._get_manifest_presign_url()
+    else:
+        presign_url = gantry_dataset_obj._get_manifest_presign_url(
+            version_id=commit_history[1]["id"]
+        )
+    assert presign_url.endswith("presign")
+
+
+@pytest.mark.parametrize(
+    ["dataset_name", "hf_name", "expected_name", "streaming"],
+    [
+        ("hf-test", "hf_name", "hf_name", False),
+        ("hf-test", None, "hftest", False),
+        ("hf-test", "hf_name", "hf_name", True),
+        ("hf-test", None, "hftest", True),
+    ],
+)
+def test_get_huggingface_dataset(
+    dataset_name, hf_name, expected_name, streaming, gantry_dataset_obj
+):
     """
     Test get local edit diff
     """
     # overwrite the dataset name
-    gantry_dataset_obj.dataset_name = "hf_test"
+    gantry_dataset_obj.dataset_name = dataset_name
 
-    hf_dataset = gantry_dataset_obj.get_huggingface_dataset()
+    hf_dataset = gantry_dataset_obj.get_huggingface_dataset(
+        hf_dataset_name=hf_name, streaming=streaming
+    )
 
-    assert len(hf_dataset["train"]) == 40
-    assert hf_dataset["train"].features == {
+    if not streaming:
+        assert len(hf_dataset) == 40
+    else:
+        len(list(hf_dataset)) == 40
+        assert hf_dataset.__class__ == datasets.iterable_dataset.IterableDataset
+
+    assert hf_dataset.features == {
         "inputs.credit_history": Value(dtype="string", id=None),
+        "inputs.credit_scores": Sequence(
+            feature=Value(dtype="float64", id=None), length=-1, id=None
+        ),
         "inputs.description": Value(dtype="string", id=None),
         "inputs.image_uri": Value(dtype="string", id=None),
         "inputs.income": Value(dtype="int64", id=None),
@@ -195,7 +426,58 @@ def test_get_huggingface_dataset(gantry_dataset_obj):
         "feedback.loan_repaid": Value(dtype="bool", id=None),
         "__time": Value(dtype="timestamp[ns, tz=UTC]", id=None),
     }
-    assert hf_dataset["train"].builder_name == "hf_test"
+    assert hf_dataset.builder_name == expected_name
+
+
+@pytest.mark.parametrize(
+    ["dataset_name", "hf_name", "load_from_remote"],
+    [
+        ("12-123", None, True),
+        ("hf_test", "123", True),
+        ("12-123", None, False),
+        ("hf_test", "123", False),
+    ],
+)
+def test_get_huggingface_dataset_error(dataset_name, hf_name, load_from_remote, gantry_dataset_obj):
+    gantry_dataset_obj.dataset_name = dataset_name
+    if load_from_remote:
+        hf_dataset = gantry_dataset_obj.pull_huggingface_dataset_by_version(hf_dataset_name=hf_name)
+    else:
+        hf_dataset = gantry_dataset_obj.get_huggingface_dataset(hf_dataset_name=hf_name)
+
+    assert hf_dataset is None
+
+
+def test_get_huggingface_dataset_no_folder(gantry_dataset_obj):
+    """
+    Test error when no tabular manifests folder
+    """
+    gantry_dataset_obj.dataset_name = "no_tabular_manifest_dataset"
+
+    with pytest.raises(NoTabularDataError):
+        gantry_dataset_obj.get_huggingface_dataset()
+
+
+def test_get_huggingface_dataset_mismatch_schema(gantry_dataset_obj):
+    """
+    Test error when no tabular manifests folder
+    """
+    gantry_dataset_obj.dataset_name = "mismatch_config_dataset"
+
+    with pytest.raises(DataSchemaMismatchError):
+        gantry_dataset_obj.get_huggingface_dataset()
+
+
+def test_get_huggingface_dataset_bad_data_type_schema(gantry_dataset_obj):
+    """
+    Test error when no tabular manifests folder
+    """
+    gantry_dataset_obj.dataset_name = "bad_data_config_dataset"
+
+    with pytest.raises(DataSchemaMismatchError) as e:
+        gantry_dataset_obj.get_huggingface_dataset()
+
+    assert isinstance(e.value.__cause__, builder.DatasetGenerationError)
 
 
 def setup_resp_for_deleted_check(resp, is_deleted=False, dataset_name=DATASET_NAME):
@@ -748,7 +1030,7 @@ def test_add_dataframe(gantry_dataset_obj, filename, expected_filename, commit_m
 
     df = pd.DataFrame({"A": [100, 101], "B": [200, 202]})
     expected_file = StringIO()
-    df.to_csv(expected_file)
+    df.to_csv(expected_file, index=False)
     expected_file.seek(0)
 
     with responses.RequestsMock() as resp:
@@ -763,6 +1045,69 @@ def test_add_dataframe(gantry_dataset_obj, filename, expected_filename, commit_m
         gantry_dataset_obj.push_dataframe(df, filename, commit_message)
 
 
+@patch("uuid.uuid4", lambda: "uuid-uuid-uuid")
+@pytest.mark.parametrize("filename", ["", "mock_filename"])
+def test_push_gantry_dataframe(gantry_dataset_obj, test_api_client, filename):
+    start_time = datetime.datetime.now()
+    end_time = datetime.datetime.now()
+    final_filename_prefixes = {
+        "": f"GantryDataAutoSave__{uuid.uuid4()}_{start_time.strftime('%Y-%m-%d_%H:%M')}_to_{end_time.strftime('%Y-%m-%d_%H:%M')}",  # noqa: E501
+        "mock_filename": "mock_filename",
+    }
+    with responses.RequestsMock() as resp:
+        resp.add(
+            resp.GET,
+            f"{HOST}/api/v1/models/foobar/schemas",
+            json={
+                "response": "ok",
+                "data": {"id": "app_model_node_id"},
+            },
+            headers={"Content-Type": "application/json"},
+        )
+        gantrydf = GantryDataFrame(
+            api_client=test_api_client,
+            application="foobar",
+            version="1.2.3",
+            env="pytest",
+            start_time=start_time,
+            end_time=end_time,
+            filters=[],
+        )
+    with responses.RequestsMock() as resp:
+        resp.add(
+            resp.POST,
+            f"{HOST}/api/v1/datasets/{DATASET_ID}/raw_data",
+            json={
+                "response": "ok",
+                "data": {
+                    "id": "default-version-id",
+                    "message": "default-message",
+                    "created_at": "creation-timestamp",
+                    "created_by": "gantry",
+                    "is_latest_commit": True,
+                },
+            },
+            headers={"Content-Type": "application/json"},
+            match=[
+                matchers.json_params_matcher(
+                    {
+                        "model_id": "app_model_node_id",
+                        "raw_data_query_parameters": {
+                            "filters": gantrydf.filters,
+                            "tags": gantrydf.tags,
+                            "include_join_id": False,
+                            "hidden_columns": None,
+                            "start_time": str(start_time.isoformat()) + "Z",
+                            "end_time": str(end_time.isoformat()) + "Z",
+                        },
+                        "raw_data_csv_prefix": final_filename_prefixes[filename],
+                    }
+                )
+            ],
+        )
+        gantry_dataset_obj.push_gantry_dataframe(gantrydf, filename)
+
+
 def update_manifest_with_checksums(datadir, dataset):
     """
     Updates the gantry manifest so that there is no local dif
