@@ -1345,14 +1345,63 @@ def test_log_records_preds_and_feedback_series_coerced(
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
 @pytest.mark.parametrize(
-    "tags, expected_tags_param",
+    "tags, row_tags, global_tags, expected_tags_param",
     [
-        (None, [{"env": "test"}, {"env": "test"}]),
-        ({"bar": "bar"}, [{"bar": "bar", "env": "test"}, {"bar": "bar", "env": "test"}]),
+        (None, None, None, [{"env": "test"}, {"env": "test"}]),
+        (
+            {"bar": "bar"},
+            None,
+            None,
+            [{"bar": "bar", "env": "test"}, {"bar": "bar", "env": "test"}],
+        ),
+        (
+            {"bar": "bar"},
+            None,
+            {"foo": "foo"},
+            [
+                {"bar": "bar", "env": "test", "foo": "foo"},
+                {"bar": "bar", "env": "test", "foo": "foo"},
+            ],
+        ),
         (
             [{"bar": "bar"}, {"bar": "baz"}],
+            None,
+            None,
             [{"bar": "bar", "env": "test"}, {"bar": "baz", "env": "test"}],
         ),
+        (
+            None,
+            [{"bar": "bar"}, {"bar": "baz"}],
+            None,
+            [{"bar": "bar", "env": "test"}, {"bar": "baz", "env": "test"}],
+        ),
+        (
+            None,
+            [{"bar": "bar"}, {"bar": "baz"}],
+            {"foo": "foo"},
+            [
+                {"bar": "bar", "env": "test", "foo": "foo"},
+                {"bar": "baz", "env": "test", "foo": "foo"},
+            ],
+        ),
+        (
+            {"foo": "foo"},
+            [{"bar": "bar"}, {"bar": "baz"}],
+            None,
+            [
+                {"bar": "bar", "env": "test", "foo": "foo"},
+                {"bar": "baz", "env": "test", "foo": "foo"},
+            ],
+        ),
+        (
+            {"bar": "bar"},
+            [{"foo": "foo"}, {"baz": "baz"}],
+            {"env": "foobar"},
+            [
+                {"bar": "bar", "env": "foobar", "foo": "foo"},
+                {"bar": "bar", "env": "foobar", "baz": "baz"},
+            ],
+        ),
     ],
 )
 @pytest.mark.parametrize(
@@ -1384,6 +1433,8 @@ def test_log_records_preds_and_feedback(
     kwargs,
     version,
     tags,
+    row_tags,
+    global_tags,
     expected_tags_param,
     test_timestamps,
     test_timestamps_list,
@@ -1404,6 +1455,8 @@ def test_log_records_preds_and_feedback(
         sort_on_timestamp=True,
         sample_rate=1,
         tags=tags,
+        row_tags=row_tags,
+        global_tags=global_tags,
         **kwargs
     ) == (None, ["12345", "67890"])
 
@@ -1610,14 +1663,63 @@ def test_log_records_preds_and_feedback_wrong_join_keys(
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
 @pytest.mark.parametrize(
-    "tags, expected_tags_param",
+    "tags, row_tags, global_tags, expected_tags_param",
     [
-        (None, [{"env": "test"}, {"env": "test"}]),
-        ({"bar": "bar"}, [{"bar": "bar", "env": "test"}, {"bar": "bar", "env": "test"}]),
+        (None, None, None, [{"env": "test"}, {"env": "test"}]),
+        (
+            {"bar": "bar"},
+            None,
+            None,
+            [{"bar": "bar", "env": "test"}, {"bar": "bar", "env": "test"}],
+        ),
+        (
+            {"bar": "bar"},
+            None,
+            {"foo": "foo"},
+            [
+                {"bar": "bar", "env": "test", "foo": "foo"},
+                {"bar": "bar", "env": "test", "foo": "foo"},
+            ],
+        ),
         (
             [{"bar": "bar"}, {"bar": "baz"}],
+            None,
+            None,
             [{"bar": "bar", "env": "test"}, {"bar": "baz", "env": "test"}],
         ),
+        (
+            None,
+            [{"bar": "bar"}, {"bar": "baz"}],
+            None,
+            [{"bar": "bar", "env": "test"}, {"bar": "baz", "env": "test"}],
+        ),
+        (
+            None,
+            [{"bar": "bar"}, {"bar": "baz"}],
+            {"foo": "foo"},
+            [
+                {"bar": "bar", "env": "test", "foo": "foo"},
+                {"bar": "baz", "env": "test", "foo": "foo"},
+            ],
+        ),
+        (
+            {"foo": "foo"},
+            [{"bar": "bar"}, {"bar": "baz"}],
+            None,
+            [
+                {"bar": "bar", "env": "test", "foo": "foo"},
+                {"bar": "baz", "env": "test", "foo": "foo"},
+            ],
+        ),
+        (
+            {"bar": "bar"},
+            [{"foo": "foo"}, {"baz": "baz"}],
+            {"env": "foobar"},
+            [
+                {"bar": "bar", "env": "foobar", "foo": "foo"},
+                {"bar": "bar", "env": "foobar", "baz": "baz"},
+            ],
+        ),
     ],
 )
 @mock.patch("gantry.logger.client._resolve_join_keys")
@@ -1637,6 +1739,8 @@ def test_log_records_preds_only(
     mock_resolve_join_keys,
     version,
     tags,
+    row_tags,
+    global_tags,
     expected_tags_param,
     test_timestamps,
     test_timestamps_list,
@@ -1656,6 +1760,8 @@ def test_log_records_preds_only(
         timestamps=test_timestamps,
         sort_on_timestamp=True,
         tags=tags,
+        row_tags=row_tags,
+        global_tags=global_tags,
         **kwargs
     ) == (None, ["12345", "67890"])
 
@@ -1719,14 +1825,63 @@ def test_log_records_preds_only(
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
 @pytest.mark.parametrize(
-    "tags, expected_tags_param",
+    "tags, row_tags, global_tags, expected_tags_param",
     [
-        (None, [{"env": "test"}, {"env": "test"}]),
-        ({"bar": "bar"}, [{"bar": "bar", "env": "test"}, {"bar": "bar", "env": "test"}]),
+        (None, None, None, [{"env": "test"}, {"env": "test"}]),
+        (
+            {"bar": "bar"},
+            None,
+            None,
+            [{"bar": "bar", "env": "test"}, {"bar": "bar", "env": "test"}],
+        ),
+        (
+            {"bar": "bar"},
+            None,
+            {"foo": "foo"},
+            [
+                {"bar": "bar", "env": "test", "foo": "foo"},
+                {"bar": "bar", "env": "test", "foo": "foo"},
+            ],
+        ),
         (
             [{"bar": "bar"}, {"bar": "baz"}],
+            None,
+            None,
             [{"bar": "bar", "env": "test"}, {"bar": "baz", "env": "test"}],
         ),
+        (
+            None,
+            [{"bar": "bar"}, {"bar": "baz"}],
+            None,
+            [{"bar": "bar", "env": "test"}, {"bar": "baz", "env": "test"}],
+        ),
+        (
+            None,
+            [{"bar": "bar"}, {"bar": "baz"}],
+            {"foo": "foo"},
+            [
+                {"bar": "bar", "env": "test", "foo": "foo"},
+                {"bar": "baz", "env": "test", "foo": "foo"},
+            ],
+        ),
+        (
+            {"foo": "foo"},
+            [{"bar": "bar"}, {"bar": "baz"}],
+            None,
+            [
+                {"bar": "bar", "env": "test", "foo": "foo"},
+                {"bar": "baz", "env": "test", "foo": "foo"},
+            ],
+        ),
+        (
+            {"bar": "bar"},
+            [{"foo": "foo"}, {"baz": "baz"}],
+            {"env": "foobar"},
+            [
+                {"bar": "bar", "env": "foobar", "foo": "foo"},
+                {"bar": "bar", "env": "foobar", "baz": "baz"},
+            ],
+        ),
     ],
 )
 @mock.patch("gantry.logger.client._resolve_join_keys", return_value=["12345", "54321"])
@@ -1744,6 +1899,8 @@ def test_log_records_feedback_only(
     mock_resolve_join_keys,
     version,
     tags,
+    row_tags,
+    global_tags,
     expected_tags_param,
     test_feedbacks,
     test_timestamps,
@@ -1760,6 +1917,8 @@ def test_log_records_feedback_only(
         timestamps=test_timestamps,
         sort_on_timestamp=True,
         tags=tags,
+        row_tags=row_tags,
+        global_tags=global_tags,
         **kwargs
     ) == (None, ["12345", "67890"])
 
