@@ -84,6 +84,12 @@ def test_build_series(df):
         assert series["proj_datanode"].dtype == "intC"
         assert series["proj_datanode"].series_type == "C"
 
+        try:
+            repr(df)
+            repr(series)
+        except Exception as e:
+            assert False, f"__repr__ failed due to: {e}"
+
 
 @mock.patch("gantry.query.main.create_view")
 def test_create_view(mock_create_view, df):
